# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

# Ubuntu System Tests
# Copyright (C) 2016 Canonical
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import fixtures

from ubuntu_system_tests.helpers.job import is_systemd_running, JobSystemD
from ubuntu_system_tests.helpers import try_import

uutoolkit_fixtures = try_import('ubuntuuitoolkit.fixture_setup')


class JobEnvironmentVariable(fixtures.Fixture):
    """ Class used to set an environment variable for a job during the
    test execution. This Fixture could use either upstart or systemd
    depending the machine/device where it is executed.
    """

    def __init__(self, global_=False, **kwargs):
        super().__init__()
        self.variables = kwargs
        self.global_ = global_

    def setUp(self):
        super().setUp()
        if is_systemd_running():
            self.useFixture(SystemDEnvironmentVariable(self.global_,
                                                       **self.variables))
        else:
            self.useFixture(uutoolkit_fixtures.InitctlEnvironmentVariable(
                self.global_, **self.variables))


class SystemDEnvironmentVariable(fixtures.Fixture):
    """Implementation of JobEnvironmentVariable fixture for systemd
    machines/devices.
    """

    def __init__(self, global_=False, **kwargs):
        super().__init__()
        self.variables = kwargs
        self.global_ = global_

    def setUp(self):
        super().setUp()
        for variable, value in self.variables.items():
            self._add_variable_cleanup(variable)
            if value is None:
                JobSystemD.unset_environment_variable(
                    variable, global_=self.global_)
            else:
                JobSystemD.set_environment_variable(
                    variable, value, global_=self.global_)

    def _add_variable_cleanup(self, variable):
        if JobSystemD.is_environment_variable(
                variable, global_=self.global_):
            original_value = JobSystemD.get_environment_variable(
                variable, global_=self.global_)
            self.addCleanup(JobSystemD.set_environment_variable, variable,
                            original_value, global_=self.global_)
        else:
            self.addCleanup(JobSystemD.unset_environment_variable, variable,
                            global_=self.global_)
